@@ -31,6 +31,7 @@
 #include <QDateTime>
 #include <QJsonDocument>
 #include <QStandardPaths>
+#include <QRandomGenerator>
 
 namespace Minuet
 {
@@ -73,14 +74,13 @@ void ExerciseController::randomlySelectExerciseOptions()
     while (!m_selectedExerciseOptions.isEmpty())
         m_selectedExerciseOptions.removeFirst();
 
-    qsrand(QDateTime::currentDateTimeUtc().toTime_t());
-
     int minNote = INT_MAX;
     int maxNote = INT_MIN;
+    auto *generator = QRandomGenerator::global();
     quint8 numberOfSelectedOptions = m_currentExercise[QStringLiteral("numberOfSelectedOptions")].toInt();
     for (quint8 i = 0; i < numberOfSelectedOptions; ++i) {
         QJsonArray exerciseOptions = QJsonObject::fromVariantMap(m_currentExercise)[QStringLiteral("options")].toArray();
-        quint8 chosenExerciseOption = qrand() % exerciseOptions.size();
+        quint8 chosenExerciseOption = generator->bounded(exerciseOptions.size());
 
         QString sequence = exerciseOptions[chosenExerciseOption].toObject()[QStringLiteral("sequence")].toString();
         foreach(const QString &additionalNote, sequence.split(' ')) {
@@ -93,7 +93,7 @@ void ExerciseController::randomlySelectExerciseOptions()
             quint8 exerciseMinRoot = exerciseRoots.first().toInt();
             quint8 exerciseMaxRoot = exerciseRoots.last().toInt();
             do
-                m_chosenRootNote = exerciseMinRoot + qrand() % (exerciseMaxRoot - exerciseMinRoot);
+                m_chosenRootNote = exerciseMinRoot + generator->bounded(exerciseMaxRoot - exerciseMinRoot);
             while (m_chosenRootNote + maxNote > 108 || m_chosenRootNote + minNote < 21);
         }
 
@@ -117,17 +117,18 @@ QJsonArray ExerciseController::exercises() const
 
 bool ExerciseController::mergeJsonFiles(const QString directoryName, QJsonObject &targetObject, bool applyDefinitionsFlag, QString commonKey, QString mergeKey)
 {
-#if defined(Q_OS_ANDROID)
     QStringList jsonDirs;
-    jsonDirs += "/data/data/org.kde.minuet/qt-reserved-files/share/minuet/" + directoryName;
+#if defined(Q_OS_ANDROID)
+    jsonDirs << "assets:/data/" + directoryName;
 #elif defined(Q_OS_WIN)
-    QStringList jsonDirs = QStandardPaths::locateAll(QStandardPaths::AppDataLocation, QStringLiteral("minuet/") + directoryName, QStandardPaths::LocateDirectory);
+    jsonDirs = QStandardPaths::locateAll(QStandardPaths::AppDataLocation, QStringLiteral("minuet/") + directoryName, QStandardPaths::LocateDirectory);
 #else
-    QStringList jsonDirs = QStandardPaths::locateAll(QStandardPaths::AppDataLocation, directoryName, QStandardPaths::LocateDirectory);
+    jsonDirs = QStandardPaths::locateAll(QStandardPaths::AppDataLocation, directoryName, QStandardPaths::LocateDirectory);
 #endif
     foreach (const QString &jsonDirString, jsonDirs) {
         QDir jsonDir(jsonDirString);
         foreach (const QString &json, jsonDir.entryList(QDir::Files)) {
+            if (!json.endsWith(QLatin1String(".json"))) break;
             QFile jsonFile(jsonDir.absoluteFilePath(json));
             if (!jsonFile.open(QIODevice::ReadOnly)) {
 #if !defined(Q_OS_ANDROID)
